use std::{collections::HashMap, sync::Arc};

use egui_winit_vulkano::egui::mutex::Mutex;
use winit::{event::ElementState, keyboard::PhysicalKey};

use super::{
    virtual_binding::VirtualBinding,
    virtual_state::{VirtualBindingState, VirtualInputState},
};

#[derive(Default)]
pub struct VirtualInput {
    states: HashMap<String, Arc<Mutex<VirtualInputState>>>,
    states_by_key: HashMap<PhysicalKey, Vec<Arc<Mutex<VirtualInputState>>>>,
    mouse_states: Vec<Arc<Mutex<VirtualInputState>>>,
}

impl std::fmt::Debug for VirtualInput {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        let mut debug = f.debug_struct("VirtualInput");

        for (name, state) in &self.states {
            debug.field(name, &state.lock().value);
        }

        debug.finish()
    }
}

impl VirtualInput {
    pub fn get_state(&self, value_name: &str) -> f32 {
        self.states
            .get(value_name)
            .and_then(|state| Some(state.lock().value))
            .unwrap_or(0.0)
    }

    pub fn add_bindings(&mut self, value_name: String, new_bindings: Vec<VirtualBinding>) {
        add_bindings(
            &mut self.states,
            &mut self.states_by_key,
            &mut self.mouse_states,
            value_name,
            new_bindings,
        );
    }

    pub fn add_binding(&mut self, value_name: String, binding: VirtualBinding) {
        self.add_bindings(value_name, vec![binding]);
    }

    pub(super) fn update_key_binding(&mut self, key: PhysicalKey, key_state: ElementState) {
        let states = self.states_by_key.get_mut(&key);

        if let Some(states) = states {
            for state in states {
                let mut state = state.lock();
                state.update_from_key(key, key_state);
            }
        } else {
            log::trace!("{}", self.states_by_key.keys().len());
            log::warn!("No states found for key: {key:?}");
        }
    }

    pub(super) fn update_mouse_binding(&mut self, delta: &glam::Vec2) {
        for state in &mut self.mouse_states {
            let mut state = state.lock();
            state.update_from_mouse(delta);
        }
    }
}

fn add_bindings(
    states: &mut HashMap<String, Arc<Mutex<VirtualInputState>>>,
    states_by_key: &mut HashMap<PhysicalKey, Vec<Arc<Mutex<VirtualInputState>>>>,
    mouse_states: &mut Vec<Arc<Mutex<VirtualInputState>>>,
    value_name: String,
    new_bindings: Vec<VirtualBinding>,
) {
    let state = states
        .entry(value_name)
        .or_insert(Arc::new(Mutex::new(VirtualInputState {
            value: 0.0,
            bindings: Vec::new(),
        })));

    for binding in &new_bindings {
        match binding {
            VirtualBinding::Keyboard(key, _) => {
                states_by_key
                    .entry(*key)
                    .or_insert(Vec::new())
                    .push(state.clone());
            }
            VirtualBinding::MouseX(_) | VirtualBinding::MouseY(_) => {
                mouse_states.push(state.clone());
            }
            _ => {}
        }
    }

    let mut state = state.lock();
    state
        .bindings
        .extend(new_bindings.iter().map(|b| VirtualBindingState {
            value: 0.0,
            binding: b.clone(),
        }));
}
