use core::input::{
    InputManager,
    virtual_binding::{AxisDirection, VirtualBinding},
};

use vulkano::device::{DeviceExtensions, DeviceFeatures};
use vulkano_util::context::{VulkanoConfig, VulkanoContext};
use winit::{
    event_loop::{ControlFlow, EventLoop},
    keyboard::{KeyCode, PhysicalKey},
};

mod core;
mod game;

fn main() {
    env_logger::init();

    let mut input_manager = InputManager::default();
    input_manager.add_virtual_bindings(
        "move_forward".to_string(),
        vec![
            VirtualBinding::Keyboard(PhysicalKey::Code(KeyCode::KeyW), AxisDirection::Positive),
            VirtualBinding::Keyboard(PhysicalKey::Code(KeyCode::KeyS), AxisDirection::Negative),
        ],
    );
    input_manager.add_virtual_bindings(
        "move_right".to_string(),
        vec![
            VirtualBinding::Keyboard(PhysicalKey::Code(KeyCode::KeyD), AxisDirection::Positive),
            VirtualBinding::Keyboard(PhysicalKey::Code(KeyCode::KeyA), AxisDirection::Negative),
        ],
    );
    input_manager.add_virtual_bindings(
        "mouse_x".to_string(),
        vec![VirtualBinding::MouseX(AxisDirection::Positive)],
    );
    input_manager.add_virtual_bindings(
        "mouse_y".to_string(),
        vec![VirtualBinding::MouseY(AxisDirection::Positive)],
    );

    let device_extensions = DeviceExtensions {
        khr_swapchain: true,
        ..Default::default()
    };

    let device_features = DeviceFeatures {
        dynamic_rendering: true,
        ..Default::default()
    };

    let vulkano_config = VulkanoConfig {
        print_device_name: true,
        device_extensions,
        device_features,
        ..Default::default()
    };

    let vulkano_context = VulkanoContext::new(vulkano_config);

    let event_loop = EventLoop::new().unwrap();
    event_loop.set_control_flow(ControlFlow::Poll);

    let mut app = core::app::App::new(vulkano_context, input_manager);

    match event_loop.run_app(&mut app) {
        Ok(_) => {}
        Err(e) => {
            log::error!("Error running old app: {e}");
        }
    }
}
